package com.sun.org.apache.xalan.internal.xsltc.compiler;

import com.sun.org.apache.xml.internal.serializer.ToHTMLStream;
import com.sun.org.apache.xml.internal.serializer.ElemDesc;
import com.sun.org.apache.bcel.internal.generic.InstructionList;
import com.sun.org.apache.bcel.internal.generic.ConstantPoolGen;
import com.sun.org.apache.bcel.internal.generic.PUSH;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.MethodGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ClassGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ErrorMsg;
import java.util.HashMap;
import java.util.Set;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.TypeCheckError;
import java.util.Iterator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.Type;
import java.util.ArrayList;
import java.util.Hashtable;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.Util;
import java.util.Map;
import java.util.List;

final class LiteralElement extends Instruction
{
    private String _name;
    private LiteralElement _literalElemParent;
    private List<SyntaxTreeNode> _attributeElements;
    private Map<String, String> _accessedPrefixes;
    private boolean _allAttributesUnique;
    
    LiteralElement() {
        this._literalElemParent = null;
        this._attributeElements = null;
        this._accessedPrefixes = null;
        this._allAttributesUnique = false;
    }
    
    public QName getName() {
        return this._qname;
    }
    
    @Override
    public void display(int indent) {
        this.indent(indent);
        Util.println("LiteralElement name = " + this._name);
        this.displayContents(indent + 4);
    }
    
    private String accessedNamespace(String prefix) {
        if (this._literalElemParent != null) {
            String result = this._literalElemParent.accessedNamespace(prefix);
            if (result != null) {
                return result;
            }
        }
        return (this._accessedPrefixes != null) ? ((String)this._accessedPrefixes.get(prefix)) : null;
    }
    
    public void registerNamespace(String prefix, String uri, SymbolTable stable, boolean declared) {
        if (this._literalElemParent != null) {
            String parentUri = this._literalElemParent.accessedNamespace(prefix);
            if (parentUri != null && parentUri.equals(uri)) {
                return;
            }
        }
        if (this._accessedPrefixes == null) {
            this._accessedPrefixes = new Hashtable();
        }
        else if (!declared) {
            String old = (String)this._accessedPrefixes.get(prefix);
            if (old != null) {
                if (old.equals(uri)) {
                    return;
                }
                prefix = stable.generateNamespacePrefix();
            }
        }
        if (!prefix.equals("xml")) {
            this._accessedPrefixes.put(prefix, uri);
        }
    }
    
    private String translateQName(QName qname, SymbolTable stable) {
        String localname = qname.getLocalPart();
        String prefix = qname.getPrefix();
        if (prefix == null) {
            prefix = "";
        }
        else if (prefix.equals("xmlns")) {
            return "xmlns";
        }
        String alternative = stable.lookupPrefixAlias(prefix);
        if (alternative != null) {
            stable.excludeNamespaces(prefix);
            prefix = alternative;
        }
        String uri = this.lookupNamespace(prefix);
        if (uri == null) {
            return localname;
        }
        this.registerNamespace(prefix, uri, stable, false);
        if (prefix != "") {
            return prefix + ":" + localname;
        }
        return localname;
    }
    
    public void addAttribute(SyntaxTreeNode attribute) {
        if (this._attributeElements == null) {
            this._attributeElements = new ArrayList(2);
        }
        this._attributeElements.add(attribute);
    }
    
    public void setFirstAttribute(SyntaxTreeNode attribute) {
        if (this._attributeElements == null) {
            this._attributeElements = new ArrayList(2);
        }
        this._attributeElements.add(0, attribute);
    }
    
    @Override
    public Type typeCheck(SymbolTable stable) throws TypeCheckError {
        if (this._attributeElements != null) {
            for (SyntaxTreeNode node : this._attributeElements) {
                node.typeCheck(stable);
            }
        }
        this.typeCheckContents(stable);
        return Type.Void;
    }
    
    public Set<Map.Entry<String, String>> getNamespaceScope(SyntaxTreeNode node) {
        Map<String, String> all = new HashMap();
        while (node != null) {
            Map<String, String> mapping = node.getPrefixMapping();
            if (mapping != null) {
                for (String prefix : mapping.keySet()) {
                    if (!all.containsKey(prefix)) {
                        all.put(prefix, mapping.get(prefix));
                    }
                    else {
                        continue;
                    }
                }
            }
            node = node.getParent();
        }
        return all.entrySet();
    }
    
    @Override
    public void parseContents(Parser parser) {
        SymbolTable stable = parser.getSymbolTable();
        stable.setCurrentNode(this);
        SyntaxTreeNode parent = this.getParent();
        if (parent != null && parent instanceof LiteralElement) {
            this._literalElemParent = (LiteralElement)parent;
        }
        this._name = this.translateQName(this._qname, stable);
        int count = this._attributes.getLength();
        for (int i = 0; i < count; ++i) {
            QName qname = parser.getQName(this._attributes.getQName(i));
            String uri = qname.getNamespace();
            String val = this._attributes.getValue(i);
            if (qname.equals(parser.getUseAttributeSets())) {
                if (!Util.isValidQNames(val)) {
                    ErrorMsg err = new ErrorMsg("INVALID_QNAME_ERR", val, this);
                    parser.reportError(3, err);
                }
                this.setFirstAttribute(new UseAttributeSets(val, parser));
            }
            else if (qname.equals(parser.getExtensionElementPrefixes())) {
                stable.excludeNamespaces(val);
            }
            else if (qname.equals(parser.getExcludeResultPrefixes())) {
                stable.excludeNamespaces(val);
            }
            else {
                String prefix = qname.getPrefix();
                if ((prefix == null || !prefix.equals("xmlns")) && (prefix != null || !qname.getLocalPart().equals("xmlns")) && (uri == null || !uri.equals("http://www.w3.org/1999/XSL/Transform"))) {
                    String name = this.translateQName(qname, stable);
                    LiteralAttribute attr = new LiteralAttribute(name, val, parser, this);
                    this.addAttribute(attr);
                    attr.setParent(this);
                    attr.parseContents(parser);
                }
            }
        }
        Set<Map.Entry<String, String>> include = this.getNamespaceScope(this);
        for (Map.Entry<String, String> entry : include) {
            String prefix2 = (String)entry.getKey();
            if (!prefix2.equals("xml")) {
                String uri2 = this.lookupNamespace(prefix2);
                if (uri2 != null && !stable.isExcludedNamespace(uri2)) {
                    this.registerNamespace(prefix2, uri2, stable, true);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        this.parseChildren(parser);
        for (int j = 0; j < count; ++j) {
            QName qname2 = parser.getQName(this._attributes.getQName(j));
            String val = this._attributes.getValue(j);
            if (qname2.equals(parser.getExtensionElementPrefixes())) {
                stable.unExcludeNamespaces(val);
            }
            else if (qname2.equals(parser.getExcludeResultPrefixes())) {
                stable.unExcludeNamespaces(val);
            }
        }
    }
    
    @Override
    protected boolean contextDependent() {
        return this.dependentContents();
    }
    
    @Override
    public void translate(ClassGenerator classGen, MethodGenerator methodGen) {
        ConstantPoolGen cpg = classGen.getConstantPool();
        InstructionList il = methodGen.getInstructionList();
        this._allAttributesUnique = this.checkAttributesUnique();
        il.append(methodGen.loadHandler());
        il.append(new PUSH(cpg, this._name));
        il.append(LiteralElement.DUP2);
        il.append(methodGen.startElement());
        for (int j = 0; j < this.elementCount(); ++j) {
            SyntaxTreeNode item = this.elementAt(j);
            if ((item instanceof Variable)) {
                item.translate(classGen, methodGen);
            }
        }
        if (this._accessedPrefixes != null) {
            for (Map.Entry<String, String> entry : this._accessedPrefixes.entrySet()) {
                String prefix = (String)entry.getKey();
                String uri = (String)entry.getValue();
                il.append(methodGen.loadHandler());
                il.append(new PUSH(cpg, prefix));
                il.append(new PUSH(cpg, uri));
                il.append(methodGen.namespace());
            }
        }
        if (this._attributeElements != null) {
            for (SyntaxTreeNode node : this._attributeElements) {
                if (!(node instanceof XslAttribute)) {
                    node.translate(classGen, methodGen);
                }
                else {
                    continue;
                }
            }
        }
        this.translateContents(classGen, methodGen);
        il.append(methodGen.endElement());
    }
    
    private boolean isHTMLOutput() {
        return this.getStylesheet().getOutputMethod() == 2;
    }
    
    public ElemDesc getElemDesc() {
        if (this.isHTMLOutput()) {
            return ToHTMLStream.getElemDesc(this._name);
        }
        return null;
    }
    
    public boolean allAttributesUnique() {
        return this._allAttributesUnique;
    }
    
    private boolean checkAttributesUnique() {
        boolean hasHiddenXslAttribute = this.canProduceAttributeNodes(this, true);
        if (hasHiddenXslAttribute) {
            return false;
        }
        if (this._attributeElements != null) {
            int numAttrs = this._attributeElements.size();
            Map<String, SyntaxTreeNode> attrsTable = null;
            int i = 0;
            while (i < numAttrs) {
                SyntaxTreeNode node = (SyntaxTreeNode)this._attributeElements.get(i);
                if ((node instanceof UseAttributeSets)) {
                    return false;
                }
                if ((node instanceof XslAttribute)) {
                    if (attrsTable == null) {
                        attrsTable = new HashMap();
                        for (int k = 0; k < i; ++k) {
                            SyntaxTreeNode n = (SyntaxTreeNode)this._attributeElements.get(k);
                            if ((n instanceof LiteralAttribute)) {
                                LiteralAttribute literalAttr = (LiteralAttribute)n;
                                attrsTable.put(literalAttr.getName(), literalAttr);
                            }
                        }
                    }
                    XslAttribute xslAttr = (XslAttribute)node;
                    AttributeValue attrName = xslAttr.getName();
                    if ((attrName instanceof AttributeValueTemplate)) {
                        return false;
                    }
                    if ((attrName instanceof SimpleAttributeValue)) {
                        SimpleAttributeValue simpleAttr = (SimpleAttributeValue)attrName;
                        String name = simpleAttr.toString();
                        if (name != null && attrsTable.get(name) != null) {
                            return false;
                        }
                        if (name != null) {
                            attrsTable.put(name, xslAttr);
                        }
                    }
                }
                ++i;
            }
        }
        return true;
    }
    
    private boolean canProduceAttributeNodes(SyntaxTreeNode node, boolean ignoreXslAttribute) {
        List<SyntaxTreeNode> contents = node.getContents();
        for (SyntaxTreeNode child : contents) {
            if ((child instanceof Text)) {
                Text text = (Text)child;
                if (text.isIgnore()) {
                    continue;
                }
                return false;
            }
            else {
                if (child instanceof LiteralElement || child instanceof ValueOf || child instanceof XslElement || child instanceof Comment || child instanceof Number || child instanceof ProcessingInstruction) {
                    return false;
                }
                if ((child instanceof XslAttribute)) {
                    if (ignoreXslAttribute) {
                        continue;
                    }
                    return true;
                }
                else {
                    if (child instanceof CallTemplate || child instanceof ApplyTemplates || child instanceof Copy || child instanceof CopyOf) {
                        return true;
                    }
                    if ((child instanceof If || child instanceof ForEach) && this.canProduceAttributeNodes(child, false)) {
                        return true;
                    }
                    if ((child instanceof Choose)) {
                        List<SyntaxTreeNode> chooseContents = child.getContents();
                        for (SyntaxTreeNode chooseChild : chooseContents) {
                            if ((chooseChild instanceof When || chooseChild instanceof Otherwise) && this.canProduceAttributeNodes(chooseChild, false)) {
                                return true;
                            }
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return false;
    }
}
